from __future__ import annotations

from abc import abstractproperty
from typing import Any, Dict, Generic, Optional, Type, TypeVar

from xcnt.sqlalchemy.search.api_gateway_crd.argument_collection_generator import ArgumentCollectionGenerator
from xcnt.sqlalchemy.search.api_gateway_crd.argument_collection_representation import ArgumentCollectionRepresentation
from xcnt.sqlalchemy.search.api_gateway_crd.fields.api_gateway_converter_plugin import (
    APIGatewayConverterPlugin,
    ConvertElement,
)
from xcnt.sqlalchemy.search.query.resource_base import ResourceBase

ReferenceTarget = TypeVar("ReferenceTarget", bound=ResourceBase)


class AbstractReferenceConverter(
    APIGatewayConverterPlugin[ConvertElement], Generic[ConvertElement, ReferenceTarget]  # type: ignore
):
    @abstractproperty
    def target(self) -> Type[ReferenceTarget]:
        pass

    @property
    def identifier(self) -> Any:
        return self.target

    def to_api_gateway_part(self) -> Optional[Dict[str, Any]]:
        resource = self.generator.collection_for(self.identifier)
        if resource is None:
            resource = self._generate_target_argument_collection()
            assert resource is not None

        reference: Dict[str, Any] = {
            "type": {
                "list": False,
                "object": True,
                "argumentCollectionRef": resource.name,
            }
        }

        return reference

    def _generate_target_argument_collection(self) -> ArgumentCollectionRepresentation:
        argument_collection_representation = ArgumentCollectionGenerator(self.target, self.generator).generate()
        return argument_collection_representation
