import io
import os
from typing import Iterable, Optional, Type, Union

from xcnt.sqlalchemy.search.api_gateway_crd.generator import Generator
from xcnt.sqlalchemy.search.api_gateway_crd.result import Result, YamlSelector
from xcnt.sqlalchemy.search.api_gateway_crd.yaml_spec import YamlSpec
from xcnt.sqlalchemy.search.query.resource_base import ResourceBase


def default_selector(spec: YamlSpec) -> bool:
    return True


class ApiGatewayCRD:
    def __init__(self, definitions: Optional[Iterable[Union[Type[ResourceBase], str]]] = None):
        self.input_data = definitions

    def build(self) -> Result:
        return Generator(self.input_data).generate()

    def as_stream(self, selector: YamlSelector = default_selector) -> io.TextIOBase:
        iodata = io.StringIO()
        self.build().dump_to(iodata, selector)
        return iodata

    def to_directory(self, directory: str, selector: YamlSelector = default_selector) -> None:
        result = self.build()
        for item in result.filtered(selector):
            full_file_path = os.path.join(directory, item.yaml_file_name)
            with open(full_file_path, "w") as yaml_handle:
                item.dump_to(yaml_handle)
