from __future__ import annotations

from typing import TYPE_CHECKING, Any, Dict, Type

from xcnt.sqlalchemy.search.api_gateway_crd.argument_collection_representation import ArgumentCollectionRepresentation
from xcnt.sqlalchemy.search.api_gateway_crd.base_resource_generator import BaseResourceGenerator
from xcnt.sqlalchemy.search.api_gateway_crd.constants import APIGATEWAY_NAME_KEY
from xcnt.sqlalchemy.search.api_gateway_crd.fields.util import api_gateway_field_query
from xcnt.sqlalchemy.search.meta_resolver import find_optional_meta_key
from xcnt.sqlalchemy.search.query.resource_base import ResourceBase
from xcnt.sqlalchemy.search.query.resource_having import DefaultHavingClass, PartialHavingBaseClass

if TYPE_CHECKING:
    from xcnt.sqlalchemy.search.api_gateway_crd.generator import Generator


def collection_name_for(item: Type[ResourceBase], generator: Generator) -> str:
    if item is DefaultHavingClass or item is PartialHavingBaseClass:
        return "having"

    api_gateway_name = find_optional_meta_key(item, APIGATEWAY_NAME_KEY)
    return generator.argument_collection_name_for(item, name=api_gateway_name)


class ArgumentCollectionGenerator(BaseResourceGenerator):
    def generate(self) -> ArgumentCollectionRepresentation:
        representation = ArgumentCollectionRepresentation(
            name=self.collection_name,
            resource=self.resource,
            spec={},
        )
        self.generator.add_argument_collection(representation)
        representation.spec = self._generate()
        return representation

    def _generate(self) -> Dict[str, Any]:
        api_spec = api_gateway_field_query.api_spec_for(self.resource, self.generator)
        assert api_spec is not None
        return api_spec

    @property
    def collection_name(self) -> str:
        return collection_name_for(self.resource, self.generator)
