from __future__ import annotations

from typing import Any, Type

from xcnt.sqlalchemy.search.api_gateway_crd.abstract_reference_converter import AbstractReferenceConverter
from xcnt.sqlalchemy.search.api_gateway_crd.argument_collection_representation import ArgumentCollectionRepresentation
from xcnt.sqlalchemy.search.api_gateway_crd.base_resource_converter import BaseResourceConverter
from xcnt.sqlalchemy.search.query import FieldGroup


class FieldGroupConverter(AbstractReferenceConverter[FieldGroup, FieldGroup]):
    @classmethod
    def supports(cls, entity: Any) -> bool:
        return isinstance(entity, FieldGroup)

    @property
    def target(self) -> Type[FieldGroup]:
        return self.item.__class__

    def _generate_target_argument_collection(self) -> ArgumentCollectionRepresentation:
        api_spec = BaseResourceConverter[FieldGroup](self.item.__class__, self.generator).to_api_gateway_part()
        assert api_spec is not None
        collection = ArgumentCollectionRepresentation(
            name=api_spec["metadata"]["name"], spec=api_spec, resource=self.item.__class__
        )
        self.generator.add_argument_collection(collection)
        return collection
