from abc import ABC
from typing import Any, Dict, Generic, Optional, Type, TypeVar

from xcnt.sqlalchemy.search.api_gateway_crd import api_gateway_type_mixin
from xcnt.sqlalchemy.search.api_gateway_crd.fields.api_gateway_converter_plugin import APIGatewayConverterPlugin
from xcnt.sqlalchemy.search.fields.function.base_function import BaseFunctionField

FunctionField = TypeVar("FunctionField", bound=BaseFunctionField)


class BaseFunctionConverter(
    APIGatewayConverterPlugin[FunctionField],
    api_gateway_type_mixin.Base,
    Generic[FunctionField],
    ABC,
):
    field_class: Type[FunctionField]

    @classmethod
    def supports(self, entity: Any) -> bool:
        return isinstance(entity, self.field_class)

    def to_api_gateway_part(self) -> Optional[Dict[str, Any]]:
        api_gateway_part: Dict[str, Any] = {}
        if self.item.description is not None:
            api_gateway_part["description"] = self.item.description
        api_gateway_part["type"] = self.to_api_gateway_type()
        return api_gateway_part
