from typing import Any, Dict, Optional

from xcnt.sqlalchemy.search.api_gateway_crd.fields.api_gateway_converter_plugin import APIGatewayConverterPlugin
from xcnt.sqlalchemy.search.api_gateway_crd.fields.scalar.argument_collection import ArgumentCollectionRequest
from xcnt.sqlalchemy.search.api_gateway_crd.fields.util import api_spec_for
from xcnt.sqlalchemy.search.fields.scalar.base_scalar import BaseScalarField


class ScalarFieldConverter(APIGatewayConverterPlugin[BaseScalarField]):
    @classmethod
    def supports(self, entity: Any) -> bool:
        return isinstance(entity, BaseScalarField)

    def to_api_gateway_part(self) -> Optional[Dict[str, Any]]:
        field_spec = {
            "required": False,
            "type": {"argumentCollectionRef": self.scalar_filter_name, "object": True, "list": False},
        }

        if self.item.description is not None:
            field_spec["description"] = self.item.description

        return field_spec

    @property
    def scalar_filter_name(self) -> str:
        return self.scalar_filter["metadata"]["name"]

    @property
    def scalar_filter(self) -> Dict[str, Any]:
        request = ArgumentCollectionRequest(field=self.item)
        filter_spec = api_spec_for(request, generator=self.generator)
        assert filter_spec is not None
        return filter_spec
