from typing import Any, Dict, NamedTuple, Optional

from xcnt.sqlalchemy.search.parser.filter_parser import FilterFieldParser
from xcnt.sqlalchemy.search.parser.filters import FilterClass


class FilterRequest(NamedTuple):
    type_name: str
    filter_name: str
    type_key: str = "scalar"

    @property
    def type_dict(self) -> Dict[str, Any]:
        return {self.type_key: self.type_name, "list": self.is_list, "object": self.is_object}

    @property
    def is_object(self) -> bool:
        return self.type_key == "argumentCollectionRef"

    @property
    def is_list(self) -> bool:
        if self.filter is None:
            return False

        return self.filter.is_listed

    @property
    def filter(self) -> Optional[FilterClass]:
        return FilterFieldParser.function_mapping.get(self.filter_name, None)
