from __future__ import annotations

from typing import TYPE_CHECKING, Any, Dict, Iterable, Optional, Type

import pkg_resources

from xcnt.sqlalchemy.search.api_gateway_crd.fields.api_gateway_converter_plugin import APIGatewayConverterPlugin

PLUGIN_ENTRY_POINT = "xcnt.sqlalchemy.search.field.apigatewaycrd"

if TYPE_CHECKING:
    from xcnt.sqlalchemy.search.api_gateway_crd.generator import Generator


class APIGatewayCRDFieldQuery:
    def __init__(self) -> None:
        self._fields: Optional[Dict[str, Type[APIGatewayConverterPlugin]]] = None

    def get(self) -> Dict[str, Type[APIGatewayConverterPlugin]]:
        if self._fields is None:
            self._fields = dict(
                (entry_point.name, entry_point.load())
                for entry_point in pkg_resources.iter_entry_points(PLUGIN_ENTRY_POINT)
            )
        assert self._fields is not None
        return self._fields

    def get_iterable(self) -> Iterable[Type[APIGatewayConverterPlugin]]:
        return self.get().values()

    def get_for(self, field: Any) -> Type[APIGatewayConverterPlugin]:
        for entity in self.get_iterable():
            if entity.supports(field):  # type: ignore
                return entity

        raise ValueError(f"Didn't find suitable api gateway crd renderer for field {field}")

    def init_for(self, field: Any, generator: Generator) -> APIGatewayConverterPlugin:
        query_class = self.get_for(field)
        return query_class(item=field, generator=generator)

    def api_spec_for(self, field: Any, generator: Generator) -> Optional[Dict[str, Any]]:
        converter = self.init_for(field, generator)
        return converter.to_api_gateway_part()

    def register(self, name: str, field: Type[APIGatewayConverterPlugin]) -> None:
        fields = self.get()
        fields[name] = field


api_gateway_field_query = APIGatewayCRDFieldQuery()

get = api_gateway_field_query.get
get_iterable = api_gateway_field_query.get_iterable
get_for = api_gateway_field_query.get_for
init_for = api_gateway_field_query.init_for
api_spec_for = api_gateway_field_query.api_spec_for
register = api_gateway_field_query.register
