import inspect
from typing import Any, Dict, Optional, Type, Union, cast

from xcnt.sqlalchemy.search.api_gateway_crd.base_resource_converter import BaseResourceConverter
from xcnt.sqlalchemy.search.api_gateway_crd.fields.util import api_gateway_field_query
from xcnt.sqlalchemy.search.api_gateway_crd.having_reference import HavingReference
from xcnt.sqlalchemy.search.api_gateway_crd.mixin import ResourceFieldsMixin
from xcnt.sqlalchemy.search.parser.having import HAVING_KEY
from xcnt.sqlalchemy.search.query import PartialResource, Resource
from xcnt.sqlalchemy.search.query.resource_having import AbstractHavingBaseClass


class ResourceConverter(BaseResourceConverter[Union[Resource, PartialResource]], ResourceFieldsMixin):
    @classmethod
    def supports(self, entity: Any) -> bool:
        return inspect.isclass(entity) and issubclass(entity, (Resource, PartialResource))

    def to_api_gateway_part(self) -> Optional[Dict[str, Any]]:
        collection = super().to_api_gateway_part()
        assert collection is not None
        collection_arguments = collection["spec"]["arguments"]
        if self.should_render_having:
            collection_arguments[HAVING_KEY] = self.render_having()

        self.add_self_nested_fields(collection_arguments)
        self.add_self_uuid_fields(collection_arguments)

        return collection

    @property
    def having(self) -> Type[AbstractHavingBaseClass]:
        return cast(Type[AbstractHavingBaseClass], self.item.having)

    @property
    def should_render_having(self) -> bool:
        return self.having is not None and cast(bool, self.item._aggregate)

    def render_having(self) -> Dict[str, Any]:
        api_spec = api_gateway_field_query.api_spec_for(HavingReference(self.having), self.generator)
        assert api_spec is not None
        return api_spec
