import itertools
from typing import IO, Callable, Iterable, List, NamedTuple, Optional

import yaml

from xcnt.sqlalchemy.search.api_gateway_crd.argument_collection_representation import ArgumentCollectionRepresentation
from xcnt.sqlalchemy.search.api_gateway_crd.enum_representation import EnumRepresentation
from xcnt.sqlalchemy.search.api_gateway_crd.yaml_spec import YamlSpec

YamlSelector = Callable[[YamlSpec], bool]


class Result(NamedTuple):
    argument_collections: List[ArgumentCollectionRepresentation]
    enums: List[EnumRepresentation]

    def get_enum(self, name: str) -> EnumRepresentation:
        for enum in self.enums:
            if enum.name == name:
                return enum

        raise KeyError(f"Enum with name {name} not found")

    def get_argument_collection(self, name: str) -> ArgumentCollectionRepresentation:
        for argument_collection in self.argument_collections:
            if argument_collection.name == name:
                return argument_collection

        raise KeyError(f"Argument Collection with name {name} not found")

    @property
    def all_specs(self) -> Iterable[YamlSpec]:
        return itertools.chain(
            self.argument_collections,
            self.enums,
        )

    def filtered(self, selector: Optional[YamlSelector] = None) -> Iterable[YamlSpec]:
        selector = selector or (lambda item: True)

        return (spec for spec in self.all_specs if selector(spec))

    def dump_to(self, stream: IO[str], selector: Optional[YamlSelector] = None) -> None:
        yaml.dump_all([item.spec for item in self.filtered(selector)], stream=stream)
