from dataclasses import dataclass
from typing import IO, Any, Dict, Generic, Optional, TypeVar

import yaml

YamlResource = TypeVar("YamlResource")


@dataclass
class YamlSpec(Generic[YamlResource]):
    name: str
    spec: Dict[str, Any]
    resource: YamlResource

    @property
    def yaml_name_prefix(self) -> Optional[str]:
        return None

    @property
    def yaml_name_suffix(self) -> Optional[str]:
        return None

    @property
    def yaml_name_extension(self) -> str:
        return ".yaml"

    @property
    def yaml_file_name(self) -> str:
        name = "-".join(item for item in (self.yaml_name_prefix, self.name, self.yaml_name_suffix) if item is not None)
        return f"{name}{self.yaml_name_extension}"

    def to_yaml(self) -> str:
        return yaml.dump(self.spec)

    def dump_to(self, stream: IO[str]) -> None:
        yaml.dump(self.spec, stream=stream)
