from typing import Any, Dict, Generic, TypeVar

from xcnt.sqlalchemy.search.apispec.fields.function_scalar_base import FunctionScalarBaseConverter
from xcnt.sqlalchemy.search.apispec.fields.scalar.filter.request import FilterRequest
from xcnt.sqlalchemy.search.apispec.fields.util import get_for
from xcnt.sqlalchemy.search.documentation_generator import ScalarFieldGenerator
from xcnt.sqlalchemy.search.fields.scalar.base_scalar import BaseScalarField

ScalarField = TypeVar("ScalarField", bound=BaseScalarField)


class ScalarFieldConverter(FunctionScalarBaseConverter[ScalarField], Generic[ScalarField], ScalarFieldGenerator):
    def to_openapi(self) -> Dict[str, Any]:
        openapi_spec = {"type": "object", "properties": self.get_filter_properties()}
        if self.item.description is not None:
            openapi_spec["description"] = self.item.description
        return openapi_spec

    def get_filter_properties(self) -> Dict[str, Any]:
        filter_properties: Dict[str, Any] = {}
        for filter_name in self.supported_filters:
            filter_properties[filter_name] = self._generate_filter(filter_name)
        filter_properties["_null"] = {"type": "boolean"}
        return filter_properties

    def _generate_filter(self, filter_name: str) -> Dict[str, Any]:
        request = FilterRequest(filter_name=filter_name, type_dict=self.type_dict)
        converter_class = get_for(request)
        filter_converter = converter_class(request, self.converter)
        return filter_converter.to_openapi()
