from typing import Any, Dict, Iterable, Optional, Type

import pkg_resources

from xcnt.sqlalchemy.search.apispec.fields.openapi_converter_plugin import OpenAPIConverterPlugin

PLUGIN_ENTRY_POINT = "xcnt.sqlalchemy.search.field.apispec"


class OpenAPIFieldQuery:
    def __init__(self) -> None:
        self._fields: Optional[Dict[str, Type[OpenAPIConverterPlugin]]] = None

    def get(self) -> Dict[str, Type[OpenAPIConverterPlugin]]:
        if self._fields is None:
            self._fields = dict(
                (entry_point.name, entry_point.load())
                for entry_point in pkg_resources.iter_entry_points(PLUGIN_ENTRY_POINT)
            )
        assert self._fields is not None
        return self._fields

    def get_iterable(self) -> Iterable[Type[OpenAPIConverterPlugin]]:
        return self.get().values()

    def get_for(self, field: Any) -> Type[OpenAPIConverterPlugin]:
        for entity in self.get_iterable():
            if entity.supports(field):  # type: ignore
                return entity

        raise ValueError(f"Didn't find suitable openapi renderer for field {field}")

    def register(self, name: str, field: Type[OpenAPIConverterPlugin]) -> None:
        fields = self.get()
        fields[name] = field


open_api_field_query = OpenAPIFieldQuery()

get = open_api_field_query.get
get_iterable = open_api_field_query.get_iterable
get_for = open_api_field_query.get_for
register = open_api_field_query.register
