from typing import Any, Dict, Union

from apispec.utils import OpenAPIVersion

from xcnt.sqlalchemy.search.apispec.openapi_converter import OpenAPIConverter


class ResourceResolver:
    def __init__(self, openapi_version: OpenAPIVersion, converter: OpenAPIConverter):
        self.openapi_version = openapi_version
        self.converter = converter

    def resolve_schema(self, data: Dict[str, Any]) -> Dict[str, Any]:
        if not isinstance(data, dict):
            return {}

        if "schema" in data:
            data["schema"] = self.resolve_schema_dict(data["schema"])

        if self.openapi_version.major >= 3:
            if "content" in data:
                for content in data["content"].values():
                    if "schema" in content:
                        content["schema"] = self.resolve_schema_dict(content["schema"])

        return data

    def resolve_schema_dict(self, schema: Union[Dict[str, Any], str]) -> Union[Dict[str, Any], str]:
        if isinstance(schema, dict):
            return self._resolve_schema_dict(schema)
        return self.converter.resolve_optional_nested_schema_reference(schema)

    def _resolve_schema_dict(self, schema: Dict[str, Any]) -> Dict[str, Any]:
        schema_type = schema.get("type", None)
        if schema_type == "array" and "items" in schema:
            schema["items"] = self.resolve_schema_dict(schema["items"])
        if schema_type == "object" and "properties" in schema:
            schema["properties"] = {
                key: self.resolve_schema_dict(definition) for key, definition in schema["properties"].items()
            }
        return schema
