from typing import Any, Dict, Optional

from apispec import APISpec, BasePlugin

from xcnt.sqlalchemy.search.apispec.openapi_converter import OpenAPIConverter
from xcnt.sqlalchemy.search.apispec.resource_resolver import ResourceResolver


class SearchParamPlugin(BasePlugin):
    Resolver = ResourceResolver
    Converter = OpenAPIConverter
    resolver: ResourceResolver
    converter: OpenAPIConverter

    def init_spec(self, spec: APISpec) -> None:
        super().init_spec(spec)
        self.spec = spec
        self.openapi_version = spec.openapi_version
        self.converter = self.Converter(openapi_version=self.openapi_version, spec=self.spec)
        self.resolver = self.Resolver(openapi_version=self.openapi_version, converter=self.converter)

    def parameter_helper(self, parameter: Dict[str, Dict[str, Any]], **kwargs: Any) -> None:
        self.resolver.resolve_schema(parameter)

    def schema_helper(self, name: str, definition: Dict[str, Any], **kwargs: Any) -> Optional[Dict[str, Any]]:
        reference = self.converter.resolve_nested_schema(name)
        if isinstance(reference, str):
            return None

        return reference
