from __future__ import annotations

import inspect
from enum import Enum
from functools import cached_property
from typing import Callable, Optional

from sqlalchemy.ext.declarative import DeclarativeMeta
from sqlalchemy.sql.elements import ColumnElement

from xcnt.sqlalchemy.search.fields.concrete_base import ConcreteBase
from xcnt.sqlalchemy.search.fields.scalar.base_scalar import BaseScalarField


class ComputationTarget(Enum):
    WHERE = "where"
    HAVING = "having"


class ComputedField(ConcreteBase):
    def __init__(
        self,
        field: BaseScalarField,
        resolver_function: Callable[[DeclarativeMeta], ColumnElement],
        attribute_name: Optional[str] = None,
        description: Optional[str] = None,
        computation_target: ComputationTarget = ComputationTarget.HAVING,
    ):
        self.field = field
        self.resolver_function = resolver_function
        self.computation_target = computation_target
        super().__init__(attribute_name=attribute_name, description=description)

    @cached_property
    def _has_resolver_context(self) -> bool:
        return "context" in inspect.getfullargspec(self.resolver_function).args
