from __future__ import annotations

from typing import TYPE_CHECKING
from weakref import ReferenceType, ref

from xcnt.sqlalchemy.search.fields.base import BaseField

if TYPE_CHECKING:
    from xcnt.sqlalchemy.search.query import Resource


class ConcreteBase(BaseField):
    _resource: ReferenceType[Resource]

    @property
    def resource(self) -> Resource:
        resource = self._resource()
        assert resource is not None
        return resource

    @resource.setter
    def resource(self, resource: Resource) -> None:
        """Setter for resource enabling garbage collection.

        Fields are cached per resource and resource instance
        in a global dictionary. This ensures that resources
        who are only referenced in the global lookup dict
        and nowhere else can get correctly garbage collected
        as there is otherwise a field -> Resource -> field loop
        which the garbage collector cannot resolve.
        """
        self._resource = ref(resource)
