from __future__ import annotations

from typing import TYPE_CHECKING, Any, Callable, Dict, List, NamedTuple, Optional, Type

from xcnt.sqlalchemy.search.fields.abstract_nested_resource import AbstractNestedResource

if TYPE_CHECKING:
    from xcnt.sqlalchemy.search.query.resource import Resource


class ResolverRequest(NamedTuple):
    attribute_key: str
    sub_resolver: ConcreteResolver
    primary_keys: List[Any]
    parent: Dict[str, Any]


def default_resolver(request: ResolverRequest) -> Dict[str, Any]:
    payload = request.parent
    payload[request.attribute_key] = {"_oneOf": request.primary_keys}
    return payload


# Doesn't support annotations configuration, that's why it requires a string
class ConcreteResolver(AbstractNestedResource["Resource"]):
    def __init__(
        self,
        resource: Type[Resource],
        resolver_function: Callable[[ResolverRequest], Dict[str, Any]] = default_resolver,
        attribute_name: Optional[str] = None,
        description: Optional[str] = None,
    ):
        """Concrete resolution of entities.

        param resolver_function: The function which is used to pass in the parent
                                 query with the primary keys which are required
        """
        self.resolver_function = resolver_function
        super().__init__(resource=resource, attribute_name=attribute_name, description=description)
