from abc import ABC
from enum import Enum
from typing import Callable, Generic, Optional

from sqlalchemy.ext.declarative import DeclarativeMeta
from sqlalchemy.sql.elements import ColumnElement

from xcnt.sqlalchemy.search.fields.function_scalar_base import ConversionTargetType, FunctionScalarBase


class FunctionTarget(Enum):
    WHERE = "where"
    GROUP_BY = "group_by"


class BaseFunctionField(FunctionScalarBase[ConversionTargetType], Generic[ConversionTargetType], ABC):  # type: ignore
    accepts_non_dict: bool = True

    def __init__(
        self,
        function: Callable[[DeclarativeMeta, ColumnElement, Optional[ConversionTargetType]], ColumnElement],
        query_field_name: Optional[str] = None,
        attribute_name: Optional[str] = None,
        description: Optional[str] = None,
        table_column_name: Optional[str] = None,
        target: FunctionTarget = FunctionTarget.WHERE,
    ):
        super().__init__(
            query_field_name=query_field_name,
            attribute_name=attribute_name,
            description=description,
            table_column_name=table_column_name,
        )
        self.function = function
        self.target = target
