from enum import Enum as EnumClass
from typing import Any, Callable, Generic, Optional, Type, TypeVar

from sqlalchemy.ext.declarative import DeclarativeMeta
from sqlalchemy.sql.elements import ColumnElement

from xcnt.sqlalchemy.search.fields.function.base_function import BaseFunctionField
from xcnt.sqlalchemy.search.type_conversion import convert_to_enum

EnumType = TypeVar("EnumType", bound=EnumClass)


class Enum(BaseFunctionField[EnumType], Generic[EnumType]):
    def __init__(
        self,
        enum: Type[EnumType],
        function: Callable[[DeclarativeMeta, ColumnElement, Optional[EnumType]], ColumnElement],
        attribute_name: Optional[str] = None,
        description: Optional[str] = None,
        table_column_name: Optional[str] = None,
    ):
        super().__init__(
            function=function,
            attribute_name=attribute_name,
            description=description,
            table_column_name=table_column_name,
        )
        self.enum = enum

    def _convert(self, value: Any) -> EnumType:
        return convert_to_enum(value, self.enum)
