from __future__ import annotations

from typing import TYPE_CHECKING, Callable, Optional, Type, Union, cast

from sqlalchemy.ext.declarative import DeclarativeMeta
from sqlalchemy.sql.elements import ClauseElement

from xcnt.sqlalchemy.search.fields.abstract_nested_resource import AbstractNestedResource
from xcnt.sqlalchemy.search.fields.concrete_base import ConcreteBase

if TYPE_CHECKING:
    from xcnt.sqlalchemy.search.query.resource import Resource


class Infer:
    pass


JoinConditionCallable = Callable[[DeclarativeMeta, DeclarativeMeta], ClauseElement]


class JoinConditionConfig(ConcreteBase):
    def __init__(
        self,
        join_condition: Union[Type[Infer], JoinConditionCallable] = Infer,
        model_attribute_name: Optional[str] = None,
        attribute_name: Optional[str] = None,
        description: Optional[str] = None,
    ) -> None:
        self.join_condition = join_condition
        self.model_attribute_name = model_attribute_name
        ConcreteBase.__init__(self, attribute_name=attribute_name, description=description)

    def run_join_condition(self, left: DeclarativeMeta, right: DeclarativeMeta) -> ClauseElement:
        join_condition = self.join_condition
        assert join_condition is not Infer, "Cannot run join condition on inferred elements"
        join_condition = cast(JoinConditionCallable, join_condition)
        return join_condition(left, right)

    @property
    def infer_join_condition(self) -> bool:
        return self.join_condition is Infer


# Doesn't support annotations configuration, that's why it requires a string
class Nested(JoinConditionConfig, AbstractNestedResource["Resource"]):
    def __init__(
        self,
        resource: Union[Type[Resource], str],
        join_condition: Union[Type[Infer], JoinConditionCallable] = Infer,
        attribute_name: Optional[str] = None,
        description: Optional[str] = None,
        model_attribute_name: Optional[str] = None,
    ):
        AbstractNestedResource.__init__(self, resource=resource, attribute_name=attribute_name, description=description)
        JoinConditionConfig.__init__(
            self,
            attribute_name=attribute_name,
            description=description,
            join_condition=join_condition,
            model_attribute_name=model_attribute_name,
        )
