from typing import Any, Callable, Dict, Optional, Type

overwrite_metadata: Dict[Type[Any], Type[Any]] = {}


def register_metadata_for(cls: Type[Any], metadata: Type[Any]) -> None:
    overwrite_metadata[cls] = metadata


def as_metadata_for(item: Type[Any]) -> Callable[[Type[Any]], Type[Any]]:
    def _wrapper(metadata_class: Type[Any]) -> Type[Any]:
        register_metadata_for(item, metadata_class)
        return metadata_class

    return _wrapper


def direct_meta_key(base: Type[object], key: str) -> Any:
    meta_config = base.__dict__.get("Meta", None)

    if meta_config is not None and hasattr(meta_config, key):
        return getattr(meta_config, key)
    raise KeyError(f'Meta key "{key}" not found')


def find_meta_key(base: Type[object], key: str) -> Any:
    for candidate in base.__mro__:

        meta_config: Type[Any]

        if base in overwrite_metadata:
            meta_config = overwrite_metadata[base]
        else:
            meta_config = getattr(candidate, "Meta", None)

        if meta_config is not None and hasattr(meta_config, key):
            return getattr(meta_config, key)

    raise KeyError(f'Meta key "{key}" not found')


def find_optional_meta_key(base: Type[object], key: str, default: Optional[Any] = None) -> Optional[Any]:
    try:
        return find_meta_key(base, key)
    except KeyError:
        return default
