from __future__ import annotations

from abc import abstractclassmethod, abstractmethod
from typing import Any, Dict, Generic, TypeVar

from xcnt.sqlalchemy.search.fields.base import BaseField
from xcnt.sqlalchemy.search.parser.evaluation_result import EvaluationResult

Field = TypeVar("Field", bound=BaseField)
QueryType = TypeVar("QueryType")


class AbstractFieldPlugin(Generic[Field, QueryType]):
    key: str
    query: QueryType
    resource_field: Field

    @abstractmethod
    def __init__(self, key: str, query: QueryType, resource_field: Field):
        pass

    @abstractclassmethod
    def supports(self, entity: BaseField) -> bool:
        pass

    @abstractmethod
    def parse(self, parse_context: Dict[str, Any]) -> EvaluationResult:
        pass


class AbstractDictFieldPlugin(AbstractFieldPlugin[Field, Dict[str, Any]], Generic[Field]):
    pass
