from __future__ import annotations

from typing import Any, Dict

from sqlalchemy import sql
from sqlalchemy.sql.elements import ClauseElement
from sqlalchemy.sql.expression import ColumnClause

from xcnt.sqlalchemy.search.fields import BaseField, ComputedField
from xcnt.sqlalchemy.search.fields.computed import ComputationTarget
from xcnt.sqlalchemy.search.fields.scalar.base_scalar import BaseScalarField
from xcnt.sqlalchemy.search.parser.filter_parser import FilterFieldParser

CONTEXT_KEY = "context"


class ComputedFieldParser(FilterFieldParser[ComputedField]):
    @classmethod
    def supports(cls, entity: BaseField) -> bool:
        return isinstance(entity, ComputedField)

    @property
    def mapping_field(self) -> BaseScalarField:
        resource_field = self.resource_field.field
        assert isinstance(resource_field, BaseScalarField)
        return resource_field

    @property
    def model_field(self) -> ColumnClause:
        resolver_function = self.resource_field.resolver_function
        kwargs: Dict[str, Any] = {}

        if self.resource_field._has_resolver_context:
            kwargs[CONTEXT_KEY] = self.parse_context

        return sql.expression.label(self.key, resolver_function(self.resource.model, **kwargs))  # type:ignore

    def add_expression(self, expression: ClauseElement) -> None:
        if self.is_group_expression:
            self.group_expressions.append(expression)
        else:
            self.expressions.append(expression)

    @property
    def is_group_expression(self) -> bool:
        return self.resource_field.computation_target == ComputationTarget.HAVING
