from __future__ import annotations

from typing import TYPE_CHECKING, Any, Dict, Type

from sqlalchemy.ext.declarative import DeclarativeMeta

from xcnt.sqlalchemy.search.fields import BaseField, ConcreteResolver
from xcnt.sqlalchemy.search.parser.evaluation_result import EvaluationResult
from xcnt.sqlalchemy.search.parser.field_plugin import NonResourceFieldPlugin

if TYPE_CHECKING:
    from xcnt.sqlalchemy.search.query import Resource


class ConcreteResolverParser(NonResourceFieldPlugin[ConcreteResolver, Dict[str, Any]]):
    @classmethod
    def supports(cls, entity: BaseField) -> bool:
        return isinstance(entity, ConcreteResolver)

    def parse(self, parse_context: Dict[str, Any]) -> EvaluationResult:
        result = self._result_from_state()
        resource_result = self.nested_resource.generate(self.query, context=parse_context)
        result = result + resource_result
        return result

    @property
    def nested_resource_class(self) -> Type[Resource]:
        return self.resource_field.nested_resource

    @property
    def model(self) -> DeclarativeMeta:
        return self.nested_resource_class.model

    @property
    def nested_resource(self) -> Resource:
        return self.nested_resource_class(self.model)
