from __future__ import annotations

from typing import Any, Dict, Mapping, Type

from xcnt.sqlalchemy.search.fields.base import BaseField
from xcnt.sqlalchemy.search.meta_resolver import find_meta_key
from xcnt.sqlalchemy.search.parser.evaluation_result import EvaluationResult, ResultCombination
from xcnt.sqlalchemy.search.parser.field_plugin import FieldPlugin
from xcnt.sqlalchemy.search.parser.resource_base import ResourceBaseParser
from xcnt.sqlalchemy.search.query import FieldGroup


def get_result_combination(resource: Type[FieldGroup]) -> ResultCombination:
    return find_meta_key(resource, "combination")


class FieldGroupParserDelegator(ResourceBaseParser[Mapping[str, Any]]):
    def __init__(self, resource_field: FieldGroup, query: Mapping[str, Any]) -> None:
        super().__init__()
        self.resource_field = resource_field
        self.query = query

    def parse(self, parse_context: Dict[str, Any]) -> EvaluationResult:
        self.parse_context = parse_context
        self._parse_fields_or_raise_nothing()
        return self._result_from_state()

    @property
    def resource_fields(self) -> Dict[str, BaseField]:
        return self.resource_field._fields

    @property
    def result_combination(self) -> ResultCombination:
        return get_result_combination(self.resource_field.__class__)


class FieldGroupParser(FieldPlugin[FieldGroup, Mapping[str, Any]]):
    @classmethod
    def supports(self, entity: BaseField) -> bool:
        return isinstance(entity, FieldGroup)

    def parse(self, parse_context: Dict[str, Any]) -> EvaluationResult:
        return FieldGroupParserDelegator(self.resource_field, self.query).parse(parse_context)
