from typing import Generic, TypeVar

from xcnt.sqlalchemy.search.fields.concrete_base import ConcreteBase
from xcnt.sqlalchemy.search.parser.abstract_field_plugin import AbstractFieldPlugin, Field, QueryType
from xcnt.sqlalchemy.search.parser.evaluation_result import EvaluationResultParser
from xcnt.sqlalchemy.search.parser.keyed_field import KeyedFieldParser, ResourceKeyedFieldParser


class NonResourceFieldPlugin(
    Generic[Field, QueryType],  # type: ignore
    AbstractFieldPlugin[Field, QueryType],
    KeyedFieldParser,
    EvaluationResultParser,
):
    def __init__(self, key: str, query: QueryType, resource_field: Field):
        self.query = query
        self.resource_field = resource_field
        KeyedFieldParser.__init__(self, key=key)
        EvaluationResultParser.__init__(self)


ConcreteField = TypeVar("ConcreteField", bound=ConcreteBase)


class FieldPlugin(
    AbstractFieldPlugin[ConcreteField, QueryType],
    ResourceKeyedFieldParser,
    Generic[ConcreteField, QueryType],  # type: ignore
):
    def __init__(self, key: str, query: QueryType, resource_field: ConcreteField):
        self.query = query
        self.resource_field = resource_field
        ResourceKeyedFieldParser.__init__(self, key=key, resource=resource_field.resource)
