from typing import Any

from sqlalchemy.sql.expression import ClauseElement

from xcnt.sqlalchemy.search.parser.filters.base import BaseFilterParser
from xcnt.sqlalchemy.search.parser.filters.exception import InvalidSearchData
from xcnt.sqlalchemy.search.parser.flow_exception import ReturnAll, ReturnNothing


class InParser(BaseFilterParser):
    class Meta:
        key = "_in"
        is_listed = True

    def apply(self, value: Any, return_null_values: bool = True) -> ClauseElement:
        if value is None:
            raise ReturnAll()
        if isinstance(value, (list, tuple)) and len(value) == 0:
            raise ReturnNothing()
        return super().apply(value, return_null_values)

    def _apply_normalized(self, value: Any) -> ClauseElement:
        return self.model_field.in_(value)

    def normalize_value(self, value: Any) -> Any:
        if not isinstance(value, (list, tuple)):
            raise InvalidSearchData("Invalid input for _in")
        return [self.field.convert(item) for item in value]
