from __future__ import annotations

from typing import Any, Dict, Generic, Optional

from xcnt.sqlalchemy.search.fields import BaseField
from xcnt.sqlalchemy.search.fields.function.base_function import BaseFunctionField, ConversionTargetType, FunctionTarget
from xcnt.sqlalchemy.search.parser.evaluation_result import EvaluationResult
from xcnt.sqlalchemy.search.parser.scalar_function import ScalarFunctionFieldParser


class FunctionFieldParser(
    ScalarFunctionFieldParser[BaseFunctionField[ConversionTargetType], Any],
    Generic[ConversionTargetType],  # type: ignore
):
    @classmethod
    def supports(cls, entity: BaseField) -> bool:
        return isinstance(entity, BaseFunctionField)

    @property
    def converted_value(self) -> Optional[ConversionTargetType]:
        return self.resource_field.convert(self.query)

    @property
    def target(self) -> FunctionTarget:
        return self.resource_field.target

    def parse(self, parse_context: Dict[str, Any]) -> EvaluationResult:
        expression = self.resource_field.function(self.model, self.model_field, self.converted_value)

        if self.target == FunctionTarget.WHERE:
            self.expressions.append(expression)
        elif self.target == FunctionTarget.GROUP_BY:
            self.group_by_clauses.append(expression)
        else:
            raise NotImplementedError("This must not happen")
        return self._result_from_state()
