from __future__ import annotations

from typing import TYPE_CHECKING, Any, Dict, Type

from xcnt.sqlalchemy.search.fields import BaseField, HavingGroup
from xcnt.sqlalchemy.search.parser.evaluation_result import EvaluationResult
from xcnt.sqlalchemy.search.parser.field_plugin import FieldPlugin

if TYPE_CHECKING:
    from xcnt.sqlalchemy.search.query import HavingBaseClass


class HavingGroupFieldParser(FieldPlugin[HavingGroup, Dict[str, Any]]):
    @classmethod
    def supports(cls, entity: BaseField) -> bool:
        return isinstance(entity, HavingGroup)

    def parse(self, parse_context: Dict[str, Any]) -> EvaluationResult:
        result = self._result_from_state()
        resource_result = self.nested_resource.generate_having_group(self.query, context=parse_context)
        result = result + resource_result
        return result

    @property
    def nested_resource_class(self) -> Type[HavingBaseClass]:
        return self.resource_field.nested_resource

    @property
    def nested_resource(self) -> HavingBaseClass:
        return self.nested_resource_class(self.resource)
