from __future__ import annotations

from typing import TYPE_CHECKING, Any, Dict

from sqlalchemy.ext.declarative import DeclarativeMeta

from xcnt.sqlalchemy.search.fields import BaseField, JoinedModel
from xcnt.sqlalchemy.search.parser.evaluation_result import EvaluationResult
from xcnt.sqlalchemy.search.parser.field import FieldParser
from xcnt.sqlalchemy.search.parser.joined_field import JoinedFieldParser

if TYPE_CHECKING:
    pass


class JoinedModelParser(JoinedFieldParser[JoinedModel]):
    @classmethod
    def supports(cls, entity: BaseField) -> bool:
        return isinstance(entity, JoinedModel)

    def _combine_results(self, nested_result: EvaluationResult) -> EvaluationResult:
        nested_result.key_name = None
        self.sub_results.append(nested_result)
        result = self._result_from_state()
        return result

    def _generate_nested(self, parse_context: Dict[str, Any]) -> EvaluationResult:
        # Need to overwrite the model to allow correct filter generation for the
        # joined element.
        resource = self.resource.copy_for_model(self.model)
        wrapped_field = self.resource_field.wraps
        wrapped_field.resource = resource
        parser = FieldParser(self.key, self.query, self.resource_field.wraps)

        return parser.parse(parse_context)

    @property
    def _non_aliased_model(self) -> DeclarativeMeta:
        return self.resource_field.model

    @property
    def base_model(self) -> DeclarativeMeta:
        return self.resource.model
