from __future__ import annotations

from typing import TYPE_CHECKING, Any, Dict, Type

from sqlalchemy.ext.declarative import DeclarativeMeta

from xcnt.sqlalchemy.search.fields import BaseField, Nested
from xcnt.sqlalchemy.search.parser.evaluation_result import EvaluationResult
from xcnt.sqlalchemy.search.parser.joined_field import JoinedFieldParser

if TYPE_CHECKING:
    from xcnt.sqlalchemy.search.query import Resource


class NestedFieldParser(JoinedFieldParser[Nested]):
    @classmethod
    def supports(cls, entity: BaseField) -> bool:
        return isinstance(entity, Nested)

    def parse(self, parse_context: Dict[str, Any]) -> EvaluationResult:
        result = self._result_from_state()
        resource_result = self.nested_resource.generate_nested(self.query, context=parse_context)
        result = result + resource_result
        return result

    def _generate_nested(self, parse_context: Dict[str, Any]) -> EvaluationResult:
        return self.nested_resource.generate_nested(self.query, context=parse_context)

    @property
    def base_model(self) -> DeclarativeMeta:
        return self.resource.model

    @property
    def nested_resource_class(self) -> Type[Resource]:
        return self.resource_field.nested_resource

    @property
    def _non_aliased_model(self) -> DeclarativeMeta:
        return self.nested_resource_class.model

    @property
    def nested_resource(self) -> Resource:
        return self.nested_resource_class(self.model)
