from __future__ import annotations

from typing import TYPE_CHECKING, Dict, Optional, Union

from sqlalchemy import Column, inspect
from sqlalchemy.ext.declarative import DeclarativeMeta
from sqlalchemy.orm.util import AliasedClass
from sqlalchemy.sql.elements import ClauseElement, ColumnClause

from xcnt.sqlalchemy.search.fields.base import BaseField

if TYPE_CHECKING:
    from xcnt.sqlalchemy.search.query import HavingBaseClass, Resource


class ResourceReferenceParserMixin:
    def __init__(self, resource: Resource):
        self.resource = resource

    @property
    def resource_fields(self) -> Dict[str, BaseField]:
        return self.resource._fields

    @property
    def model(self) -> Union[DeclarativeMeta, AliasedClass]:
        return self.resource.model

    def get_column(self, column_name: str) -> Optional[ClauseElement]:
        return self.resource.get_column(column_name)

    @property
    def having(self) -> HavingBaseClass:
        return self.resource.having

    @property
    def having_fields(self) -> Dict[str, BaseField]:
        return self.having._fields

    @property
    def _primary_key_column(self) -> ColumnClause:
        return self.resource.primary_key_column

    @property
    def _primary_key_column_name(self) -> str:
        return self.resource.primary_key_column_name

    @property
    def root_model(self) -> DeclarativeMeta:
        model = self.model
        if isinstance(model, AliasedClass):
            return inspect(model).mapper.class_
        else:
            return model

    @property
    def _root_model_primary_key_column(self) -> Column:
        return getattr(self.root_model, self._primary_key_column_name)
