from typing import Any, Callable, Dict, Mapping, Optional, Type

from xcnt.sqlalchemy.search.meta_resolver import find_meta_key
from xcnt.sqlalchemy.search.parser.partial import ParsingResult, QueryParser
from xcnt.sqlalchemy.search.query.resource_base import ResourceBase, ResourceBaseMeta
from xcnt.sqlalchemy.search.query.resource_having import PartialHavingBaseClass


class PartialResourceMeta(ResourceBaseMeta):
    _meta_key: Callable[[Any, str], str]

    @property
    def having(self) -> Type[PartialHavingBaseClass]:
        return self._meta_key("having")  # type: ignore

    @property
    def _aggregate(self) -> bool:
        return self._meta_key("aggregate")  # type: ignore


class PartialResource(ResourceBase, metaclass=PartialResourceMeta):
    class Meta:
        abstract = True
        aggregate = False
        having: Type[PartialHavingBaseClass] = PartialHavingBaseClass

    def __init__(self) -> None:
        super().__init__()
        having_class = self._meta_key("having")
        self.having = having_class()
        aggregate = self._meta_key("aggregate")
        self._aggregate = aggregate

    @classmethod
    def _meta_key(self, key: str) -> Any:
        return find_meta_key(self, key)

    def generate(self, query: Dict[str, Any], context: Optional[Dict[str, Any]]) -> ParsingResult:
        query_parser = QueryParser(resource=self, query=query)
        return query_parser.parse(context)

    def generate_partial_resolved_dict(
        self, query: Dict[str, Any], context: Optional[Dict[str, Any]]
    ) -> Mapping[str, Any]:
        return self.generate(query, context).result
