from __future__ import annotations

from typing import TYPE_CHECKING, Any, Dict, Optional

from sqlalchemy import func
from sqlalchemy.ext.declarative import DeclarativeMeta

from xcnt.sqlalchemy.search import fields
from xcnt.sqlalchemy.search.parser import EvaluationResult, HavingParser
from xcnt.sqlalchemy.search.query.resource_base import ResourceBase

if TYPE_CHECKING:
    from xcnt.sqlalchemy.search.query.resource import Resource


class AbstractHavingBaseClass(ResourceBase):
    pass


class HavingBaseClass(AbstractHavingBaseClass):
    def __init__(self, resource: Resource):
        super().__init__()
        self.resource = resource

    @property
    def model(self) -> DeclarativeMeta:
        return self.resource.model

    def generate_having_group(
        self, query_filter: Dict[str, Any], context: Optional[Dict[str, Any]] = None
    ) -> EvaluationResult:
        context = context or {}
        query_parser = HavingParser(query_filter, self, self.resource)
        return query_parser.parse(context)


class DefaultHavingClass(HavingBaseClass):
    count = fields.ComputedField(fields.Integer(), resolver_function=lambda _: func.count())


class PartialHavingBaseClass(AbstractHavingBaseClass):
    count = fields.Integer()
