import os

import pytest
import yaml

from xcnt.sqlalchemy.search.api_gateway_crd import ApiGatewayCRD
from xcnt.sqlalchemy.search.api_gateway_crd.constants import (
    API_VERSION,
    APIGATEWAY_ENTITY_LABEL_KEY,
    APIGATEWAY_ROLE_LABEL_KEY,
    ARGUMENT_COLLECTION_KIND,
)
from xcnt.sqlalchemy.search.test.query import AllScalarValues


@pytest.fixture
def api_gateway_crd() -> ApiGatewayCRD:
    return ApiGatewayCRD([AllScalarValues])


def test_dump_to_directory(tmpdir, api_gateway_crd: ApiGatewayCRD):
    api_gateway_crd.to_directory(tmpdir)


def test_dump_all(api_gateway_crd: ApiGatewayCRD):
    stream = api_gateway_crd.as_stream()
    stream.read()


def test_dump_to_directory_with_selector(tmpdir, api_gateway_crd: ApiGatewayCRD):
    api_gateway_crd.to_directory(tmpdir, selector=lambda value: value.resource is AllScalarValues)
    items = os.listdir(tmpdir)
    assert len(items) == 1
    with open(os.path.join(tmpdir, items[0])) as handle:
        data = yaml.load(handle, Loader=yaml.Loader)
    assert data["apiVersion"] == API_VERSION
    assert data["kind"] == ARGUMENT_COLLECTION_KIND
    expected_field_names = AllScalarValues()._fields.keys()
    expected_additional_fields = 4
    arguments = data["spec"]["arguments"]
    assert len(expected_field_names) + expected_additional_fields == len(arguments)
    for expected_field_name in expected_field_names:
        assert expected_field_name in arguments


def test_label_set(api_gateway_crd: ApiGatewayCRD):
    result = api_gateway_crd.build()
    argument_collection = result.get_argument_collection("all-scalar-values-query")
    metadata = argument_collection.spec["metadata"]
    labels = metadata["labels"]
    assert labels[APIGATEWAY_ENTITY_LABEL_KEY] == "AllValues"
    assert labels[APIGATEWAY_ROLE_LABEL_KEY] == "WHERE"
