import pytest

from xcnt.sqlalchemy.search.api_gateway_crd import Generator
from xcnt.sqlalchemy.search.test import query


def test_having_datapoint():
    result = Generator([query.DatapointFloat]).generate()

    argument_collection = result.get_argument_collection("datapoint-float-query")
    assert (
        argument_collection.spec["spec"]["arguments"]["_having"]["type"]["argumentCollectionRef"] == "having-datapoint"
    )

    having_argument_collection = result.get_argument_collection("having-datapoint")
    having_arguments = having_argument_collection.spec["spec"]["arguments"]
    assert having_arguments["count"]["type"]["argumentCollectionRef"] == "int-query-field"
    assert having_arguments["sum"]["type"]["argumentCollectionRef"] == "float-query-field"


@pytest.mark.parametrize("key", ["_and", "_or", "_oneOf", "_noneOf"])
def test_having_resource(key: str):
    result = Generator([query.DatapointFloat]).generate()

    argument_collection = result.get_argument_collection("having-datapoint")

    assert key not in argument_collection.spec["spec"]["arguments"]


def test_having_in_subresource():
    result = Generator([query.System]).generate()

    system_argument_collection = result.get_argument_collection("system-query")
    assert system_argument_collection.spec["spec"]["arguments"].get("_having", None) is None

    component_argument_collection = result.get_argument_collection("nested-component-query")
    assert (
        component_argument_collection.spec["spec"]["arguments"]["_having"]["type"]["argumentCollectionRef"] == "having"
    )


def test_basic_having_link():
    result = Generator([query.Component]).generate()
    having_argument_collection = result.get_argument_collection("having")
    having_arguments = having_argument_collection.spec["spec"]["arguments"]
    assert having_arguments["count"]["type"]["argumentCollectionRef"] == "int-query-field"


def test_having_component_subresource_and_root_resource():
    result = Generator([query.System, query.Component]).generate()
    argument_collection = result.get_argument_collection("nested-component-query")
    assert argument_collection.spec["spec"]["arguments"]["_having"]["type"]["argumentCollectionRef"] == "having"


def test_nested_reference_to_root():
    result = Generator([query.System, query.Component]).generate()
    argument_collection = result.get_argument_collection("nested-component-query")

    assert "component-query" in argument_collection.spec["spec"]["argumentCollections"]


@pytest.mark.parametrize("key", ["_and", "_or"])
def test_nested_query_self_reference(key: str):
    result = Generator([query.System, query.Component]).generate()
    argument_collection = result.get_argument_collection("nested-component-query")

    arguments = argument_collection.spec["spec"]["arguments"]
    assert key in arguments

    assert arguments[key]["type"]["argumentCollectionRef"] == "nested-component-query"


def test_having_datapoint_having_group():
    result = Generator([query.DatapointFloat]).generate()

    argument_collection = result.get_argument_collection("datapoint-float-query")
    assert (
        argument_collection.spec["spec"]["arguments"]["_having"]["type"]["argumentCollectionRef"] == "having-datapoint"
    )

    having_argument_collection = result.get_argument_collection("having-datapoint")
    assert (
        having_argument_collection.spec["spec"]["arguments"]["sum"]["type"]["argumentCollectionRef"]
        == "float-query-field"  # noqa: W503
    )
    assert (
        having_argument_collection.spec["spec"]["arguments"]["nested_having_group"]["type"]["argumentCollectionRef"]
        == "having-aggregate-datapoint"  # noqa: W503
    )
    assert (
        having_argument_collection.spec["spec"]["arguments"]["count"]["type"]["argumentCollectionRef"]
        == "int-query-field"  # noqa: W503
    )

    having_group_argument_collection = result.get_argument_collection("having-aggregate-datapoint")
    assert (
        having_group_argument_collection.spec["spec"]["arguments"]["avg"]["type"]["argumentCollectionRef"]
        == "float-query-field"  # noqa: W503
    )
