from typing import Any, Dict, Iterable, Optional, Type

import pytest

from xcnt.sqlalchemy.search.query.resource_base import ResourceBase
from xcnt.sqlalchemy.search.test import query


@pytest.fixture
def definitions() -> Optional[Iterable[Type[ResourceBase]]]:
    return [
        query.AllFunctions,
    ]


def test_function_value(get_generated_result):
    result = get_generated_result()
    for argument_collection in result.argument_collections:
        argument_collection.to_yaml()


def test_functions(get_generated_result):
    result = get_generated_result()
    all_functions = result.get_argument_collection("all-functions-query")
    assert len(all_functions.spec["spec"]["arguments"]) >= len(query.AllFunctions._fields)


@pytest.mark.parametrize(
    "field_name,type_dict",
    [
        ["blob", {"type": {"scalar": "blob"}, "in": "query"}],
        ["boolean", {"type": {"scalar": "bool"}, "in": "query"}],
        ["date_time", {"type": {"scalar": "date_time"}, "in": "query", "description": "a datetime"}],
        ["date", {"type": {"scalar": "date"}, "in": "query", "description": "a date"}],
        ["enum", {"type": {"enumRef": "test-enum"}, "in": "query"}],
        ["float", {"type": {"scalar": "float"}, "in": "query"}],
        ["integer", {"type": {"scalar": "int"}, "in": "query"}],
        ["string", {"type": {"scalar": "string"}, "in": "query"}],
        ["time", {"type": {"scalar": "time"}, "in": "query"}],
        ["timespan", {"type": {"scalar": "timespan"}, "in": "query"}],
        ["uuid", {"type": {"scalar": "uuid"}, "in": "query"}],
    ],
)
def test_types(get_generated_result, field_name: str, type_dict: Dict[str, Any]):
    result = get_generated_result()
    all_functions = result.get_argument_collection("all-functions-query")

    assert all_functions.spec["spec"]["arguments"][field_name] == type_dict
