from typing import Iterable, Optional, Type

import pytest

from xcnt.sqlalchemy.search.query.resource_base import ResourceBase
from xcnt.sqlalchemy.search.test import query


@pytest.fixture
def definitions() -> Optional[Iterable[Type[ResourceBase]]]:
    return [
        query.AllScalarValues,
    ]


def test_scalar_value(get_generated_result):
    result = get_generated_result()
    for argument_collection in result.argument_collections:
        argument_collection.to_yaml()


@pytest.mark.parametrize("parameter", ["in_", "nin_"])
def test_scalar_value_is_listed_for(parameter, get_generated_result):
    result = get_generated_result()
    for argument_collection in result.argument_collections:
        arguments = argument_collection.spec["spec"]["arguments"]
        if parameter in arguments:
            assert arguments[parameter]["list"] is True


@pytest.mark.parametrize(
    "query_field_name",
    [
        "blob-query-field",
        "bool-query-field",
        "date-query-field",
        "date-time-query-field",
        "float-query-field",
        "int-query-field",
        "time-query-field",
        "string-query-field",
        "uuid-query-field",
    ],
)
def test_scalar_null_comparison(get_generated_result, query_field_name):
    result = get_generated_result()
    query_field = result.get_argument_collection(query_field_name)
    arguments = query_field.spec["spec"]["arguments"]
    assert "_null" in arguments
    assert arguments["_null"]["type"]["scalar"] == "bool"


def test_scalar_null_comparison_for_enums(get_generated_result):
    result = get_generated_result()
    query_field = result.get_argument_collection("test-enum-query-field")
    arguments = query_field.spec["spec"]["arguments"]
    assert "_null" in arguments
    assert arguments["_null"]["type"]["scalar"] == "bool"
