from enum import Enum
from typing import Iterable, Optional, Type

import pytest

from xcnt.sqlalchemy.search.api_gateway_crd import Generator
from xcnt.sqlalchemy.search.api_gateway_crd.fields import util
from xcnt.sqlalchemy.search.query.resource_base import ResourceBase
from xcnt.sqlalchemy.search.test import query


@pytest.fixture
def definitions() -> Optional[Iterable[Type[ResourceBase]]]:
    return [
        query.StatusObjectResource,
    ]


class Enum1(Enum):
    A = "A"


class Enum2(Enum):
    B = "B"


def test_multiple_enum():
    generator = Generator()
    enum1 = util.api_spec_for(Enum1, generator=generator)
    enum2 = util.api_spec_for(Enum2, generator=generator)
    assert enum1["metadata"]["name"] != enum2["metadata"]["name"]


def test_multiple_enum_query_fields(get_generated_result):
    result = get_generated_result()
    collection_1 = result.get_argument_collection("proposal-type-query-field")
    collection_2 = result.get_argument_collection("status-query-field")
    assert collection_1.spec["metadata"]["name"] != collection_2.spec["metadata"]["name"]


def test_multiple_enum_references(get_generated_result):
    result = get_generated_result()
    object_resource = result.get_argument_collection("status-object-resource")
    arguments = object_resource.spec["spec"]["arguments"]
    proposal_ref = arguments["proposal_type"]["type"]["argumentCollectionRef"]
    status_ref = arguments["status"]["type"]["argumentCollectionRef"]

    assert proposal_ref != status_ref
