from apispec import APISpec
from apispec.utils import build_reference

from xcnt.sqlalchemy.search.documentation_generator import SelfReferenceKey
from xcnt.sqlalchemy.search.documentation_generator.uuid_reference_key import UUIDReferenceKey
from xcnt.sqlalchemy.search.test import query


def test_all_scalar_values(api_spec: APISpec, get_parameters) -> None:
    parameter = {"content": {"application/json": {"schema": query.AllScalarValues.__name__}}}
    api_spec.components.parameter("where", "query", parameter)
    params = get_parameters()
    schema_reference = params["where"]["content"]["application/json"]["schema"]
    assert schema_reference == build_reference("schema", api_spec.openapi_version.major, query.AllScalarValues.__name__)

    components_dict = api_spec.components.to_dict()
    assert query.AllScalarValues.__name__ in components_dict["schemas"]
    schema = components_dict["schemas"][query.AllScalarValues.__name__]
    # fields, _and and _or, _one_of and _all_of
    expected_size = len(query.AllScalarValues()._fields) + len(SelfReferenceKey) + len(UUIDReferenceKey)
    assert len(schema["properties"]) == expected_size


def test_all_scalar_null_comparison_values(api_spec: APISpec, get_parameters) -> None:
    parameter = {"content": {"application/json": {"schema": query.AllScalarValues.__name__}}}
    api_spec.components.parameter("where", "query", parameter)
    params = get_parameters()
    schema_reference = params["where"]["content"]["application/json"]["schema"]
    assert schema_reference == build_reference("schema", api_spec.openapi_version.major, query.AllScalarValues.__name__)

    components_dict = api_spec.components.to_dict()
    assert query.AllScalarValues.__name__ in components_dict["schemas"]
    schema = components_dict["schemas"][query.AllScalarValues.__name__]

    for key, value in schema["properties"].items():
        if key in ["blob", "boolean", "date", "date_time", "float", "integer", "string", "time", "uuid"]:
            assert value["properties"]["_null"] == {"type": "boolean"}


def test_all_scalar_null_comparison_values_for_enum(api_spec: APISpec, get_parameters) -> None:
    parameter = {"content": {"application/json": {"schema": query.AllScalarValues.__name__}}}
    api_spec.components.parameter("where", "query", parameter)
    params = get_parameters()
    schema_reference = params["where"]["content"]["application/json"]["schema"]
    assert schema_reference == build_reference("schema", api_spec.openapi_version.major, query.AllScalarValues.__name__)

    components_dict = api_spec.components.to_dict()
    assert query.AllScalarValues.__name__ in components_dict["schemas"]
    schema = components_dict["schemas"][query.AllScalarValues.__name__]

    value = schema["properties"]["enum"]
    assert value["properties"]["_null"] == {"type": "boolean"}
