from typing import Iterable, Optional, Type

import pytest
from apispec import APISpec
from apispec.utils import build_reference

from xcnt.sqlalchemy.search.query.resource_base import ResourceBase
from xcnt.sqlalchemy.search.test import query


@pytest.fixture
def definitions() -> Optional[Iterable[Type[ResourceBase]]]:
    return [query.ComponentWithJoinedModels]


def test_component_model_with_joined_models(api_spec: APISpec, get_parameters) -> None:
    parameter = {"content": {"application/json": {"schema": query.ComponentWithJoinedModels.__name__}}}
    api_spec.components.parameter("where", "query", parameter)
    params = get_parameters()
    schema_reference = params["where"]["content"]["application/json"]["schema"]
    assert schema_reference == build_reference(
        "schema", api_spec.openapi_version.major, query.ComponentWithJoinedModels.__name__
    )

    components_dict = api_spec.components.to_dict()
    assert query.ComponentWithJoinedModels.__name__ in components_dict["schemas"]
    schema = components_dict["schemas"][query.ComponentWithJoinedModels.__name__]
    assert schema["properties"]["system_alias"]["type"] == "object"
    assert "_eq" in schema["properties"]["system_alias"]["properties"]
    assert "_like" in schema["properties"]["system_alias"]["properties"]
    assert "_ilike" in schema["properties"]["system_alias"]["properties"]
