import os

import pytest
from dotenv import find_dotenv, load_dotenv
from sqlalchemy import create_engine
from sqlalchemy.orm import close_all_sessions

from xcnt.sqlalchemy.search.test import factories as factories_module
from xcnt.sqlalchemy.search.test.model import Base, DBStatusEnum
from xcnt.sqlalchemy.search.test.session import Session


@pytest.fixture(scope="session")
def engine():
    try:
        load_dotenv(find_dotenv())
    except OSError:
        pass

    engine = create_engine(os.getenv("SQLALCHEMY_URL"), echo=False)
    try:
        Base.metadata.drop_all(engine)
    except Exception:
        pass

    try:
        DBStatusEnum.drop(engine, checkfirst=False)
    except Exception:
        pass
    Base.metadata.create_all(engine)
    Session.configure(bind=engine)
    yield engine
    engine.dispose()


@pytest.fixture
def clean_tables(engine):
    tables = Base.metadata.sorted_tables
    if len(tables) > 0:
        engine.execute(";".join(f"DELETE FROM {table.name}" for table in reversed(tables)))


@pytest.fixture
def session(engine, clean_tables):
    yield Session
    close_all_sessions()


@pytest.fixture
def factories(session):
    return factories_module
