import os
from datetime import datetime
from uuid import uuid4

import factory
from factory.fuzzy import FuzzyFloat, FuzzyText

from xcnt.sqlalchemy.search.test.factories.base import BaseFactory
from xcnt.sqlalchemy.search.test.factories.component import ComponentFactory
from xcnt.sqlalchemy.search.test.model import DatapointBlob, DatapointFloat, DatapointString


class DatapointBaseFactory(BaseFactory):
    class Meta:
        abstract = True

    uuid = factory.LazyFunction(uuid4)
    time = factory.LazyFunction(datetime.now)
    component = factory.SubFactory(ComponentFactory)


class DatapointStringFactory(DatapointBaseFactory):
    class Meta:
        model = DatapointString

    value = FuzzyText(length=20)


class DatapointFloatFactory(DatapointBaseFactory):
    class Meta:
        model = DatapointFloat

    value = FuzzyFloat(-100.0, 100.0)


class DatapointBlobFactory(DatapointBaseFactory):
    class Meta:
        model = DatapointBlob

    value = factory.LazyFunction(lambda: os.urandom(256))
