import functools
from typing import Any, Dict

import pytest
import sqlalchemy

from xcnt.sqlalchemy.search.fields import ComputationTarget, ComputedField, String
from xcnt.sqlalchemy.search.parser import FieldParser
from xcnt.sqlalchemy.search.query.resource import Resource
from xcnt.sqlalchemy.search.test import model


@pytest.fixture
def field_parser_key() -> str:
    return "my_field"


@pytest.fixture
def field_parser_query() -> Dict[str, Any]:
    return {"_eq": "abc"}


@pytest.fixture
def computation_field_factory():
    return functools.partial(ComputedField, String(), resolver_function=lambda model: sqlalchemy.true())


def test_parsing_of_computed_field_with_target_where(computation_field_factory, field_parser_key, field_parser_query):
    field = computation_field_factory(computation_target=ComputationTarget.WHERE)
    resource = Resource(model.System)
    field.resource = resource
    field_parser = FieldParser(field_parser_key, field_parser_query, field)
    result = field_parser.parse({})
    assert len(result.expressions) == 1
    assert len(result.group_expressions) == 0
    assert len(result.children) == 0


def test_parsing_of_computed_field_with_target_having(computation_field_factory, field_parser_key, field_parser_query):
    field = computation_field_factory(computation_target=ComputationTarget.HAVING)
    resource = Resource(model.System)
    field.resource = resource
    field_parser = FieldParser(field_parser_key, field_parser_query, field)
    result = field_parser.parse({})
    assert len(result.expressions) == 0
    assert len(result.group_expressions) == 1
    assert len(result.children) == 0
