from xcnt.sqlalchemy.search import fields, query
from xcnt.sqlalchemy.search.parser.evaluation_result import ResultCombination
from xcnt.sqlalchemy.search.test.model import Component, System
from xcnt.sqlalchemy.search.test.query.component_model import ComponentModelQuery
from xcnt.sqlalchemy.search.test.query.datapoint_float import DatapointAggregateGroup
from xcnt.sqlalchemy.search.test.query.system import SystemQuery


class ComponentFieldGroup(query.FieldGroup):
    code = fields.String()


class OrComponentFieldgroup(query.FieldGroup):
    class Meta:
        combination = ResultCombination.OR

    code = fields.String()
    other_code = fields.String(attribute_name="code")


class ComponentQuery(query.Resource):
    class Meta:
        model = Component

    code = fields.String()
    system = fields.Nested(SystemQuery)
    component_model = fields.Nested(ComponentModelQuery)
    created_at = fields.DateTime()
    float_datapoints = fields.Nested("xcnt.sqlalchemy.search.test.query.DatapointFloat")
    float_datapoints_via_other_name = fields.Nested(
        "xcnt.sqlalchemy.search.test.query.DatapointFloat", model_attribute_name="float_datapoints"
    )
    aggregate = DatapointAggregateGroup()
    component_attributes = ComponentFieldGroup()
    component_or_attributes = OrComponentFieldgroup()


class ComponentWithJoinedModelsQuery(query.Resource):
    class Meta:
        model = Component

    code = fields.String()
    system_alias = fields.JoinedModel(
        System,
        fields.String(table_column_name="alias"),
        join_condition=lambda component, system: component.system_uuid == system.uuid,
    )
    round_robin_component_code = fields.JoinedModel(
        System,
        fields.JoinedModel(
            Component,
            fields.String(table_column_name="code"),
            join_condition=lambda system, component: component.system_uuid == system.uuid,
        ),
        join_condition=lambda component, system: component.system_uuid == system.uuid,
    )
