from typing import Optional

from sqlalchemy.ext.declarative import DeclarativeMeta
from sqlalchemy.sql.elements import ColumnClause

from xcnt.sqlalchemy.search import fields, query
from xcnt.sqlalchemy.search.fields.function.base_function import FunctionTarget
from xcnt.sqlalchemy.search.parser import ReturnAll
from xcnt.sqlalchemy.search.test.model import Component
from xcnt.sqlalchemy.search.test.query.system import SystemQuery


def group_by_system(model: DeclarativeMeta, column: ColumnClause, value: Optional[bool]) -> ColumnClause:
    if not value:
        raise ReturnAll()

    return column


class ComponentGroupQuery(query.Resource):
    class Meta:
        model = Component

    code = fields.String()
    system = fields.Nested(SystemQuery)
    group_by_system = fields.BooleanFunction(
        group_by_system, attribute_name="system_uuid", target=FunctionTarget.GROUP_BY
    )
