from sqlalchemy import func

from xcnt.sqlalchemy.search import FieldGroup, fields, query
from xcnt.sqlalchemy.search.parser.evaluation_result import ResultCombination
from xcnt.sqlalchemy.search.test.model import DatapointFloat


class HavingAggregateDatapoint(query.HavingBaseClass):
    avg = fields.ComputedField(fields.Float(), resolver_function=lambda model: func.avg(model.value))


class HavingDatapoint(query.DefaultHavingClass):
    sum = fields.ComputedField(fields.Float(), resolver_function=lambda model: func.sum(model.value))
    nested_having_group = fields.HavingGroup(HavingAggregateDatapoint)


class FloatAggregates(FieldGroup):
    min = fields.ComputedField(fields.Float(), resolver_function=lambda model: func.min(model.value))


class DatapointAggregateValue(query.FieldGroup):
    class Meta:
        combination = ResultCombination.OR

    float = FloatAggregates()


class DatapointFloatQuery(query.Resource):
    class Meta:
        model = DatapointFloat
        aggregate = True
        having = HavingDatapoint

    value = fields.Float(table_column_name="value_float")
    aggregate_value = DatapointAggregateValue()
    component = fields.Nested("xcnt.sqlalchemy.search.test.query.Component")


class DatapointAggregateGroup(query.FieldGroup):
    float = fields.Nested(DatapointFloatQuery, model_attribute_name="float_datapoints")
