from typing import Any, Dict

from xcnt.sqlalchemy.search import fields, query
from xcnt.sqlalchemy.search.query.resource_having import PartialHavingBaseClass
from xcnt.sqlalchemy.search.test.query.component_model import ComponentModelQuery
from xcnt.sqlalchemy.search.test.query.system import SystemQuery


class PartialComponentQuery(query.PartialResource):
    system = fields.ConcreteResolver(SystemQuery)
    component_model = fields.ConcreteResolver(ComponentModelQuery)
    code = fields.String()


def resolve_for_uuid(resolver_request: fields.ResolverRequest) -> Dict[str, Any]:
    payload = resolver_request.parent
    key = resolver_request.attribute_key
    payload[f"{key}_uuid"] = {"_in": resolver_request.primary_keys}
    return payload


class PartialComponentWithSpecialKeyQuery(query.PartialResource):
    system = fields.ConcreteResolver(SystemQuery, resolver_function=resolve_for_uuid)
    component_model = fields.ConcreteResolver(ComponentModelQuery)


class InvalidPartialHavinglass(PartialHavingBaseClass):
    component_model = fields.ConcreteResolver(ComponentModelQuery)


class InvalidAggregationPartialComponentQuery(query.PartialResource):
    class Meta:
        aggregate = True
        having = InvalidPartialHavinglass

    system = fields.ConcreteResolver(SystemQuery)
    component_model = fields.ConcreteResolver(ComponentModelQuery)
    code = fields.String()
