from datetime import datetime
from typing import Optional

from sqlalchemy.ext.declarative import DeclarativeMeta
from sqlalchemy.sql.elements import ClauseElement, ColumnElement

from xcnt.sqlalchemy.search import fields, query
from xcnt.sqlalchemy.search.parser.flow_exception import ReturnAll
from xcnt.sqlalchemy.search.test.enum import StatusEnum
from xcnt.sqlalchemy.search.test.model import System
from xcnt.sqlalchemy.search.test.query.component_model import AliasGroup


class SystemQuery(query.Resource):
    class Meta:
        model = System

    alias = fields.String()
    description = fields.String()
    name = fields.String()
    status = fields.Enum(StatusEnum)
    created_at = fields.DateTime()
    components = fields.Nested("xcnt.sqlalchemy.search.test.query.Component")
    alias_group = AliasGroup()


def get_fake_column(model: DeclarativeMeta, column_name: str) -> Optional[ClauseElement]:
    if not column_name.startswith("fake_") and column_name != "uuid":
        return None

    if column_name.startswith("fake_"):
        column_name = column_name[5:]

    return getattr(model, column_name, None)


class FakeSystemQuery(query.Resource):
    class Meta:
        model = System
        clause_element_getter = get_fake_column

    fake_alias = fields.String()
    description = fields.String()


class IgnoredSystemQuery(query.Resource):
    class Meta:
        model = System
        ignored_fields = ["description"]

    alias = fields.String()
    description = fields.String()


def start_time_function(model: DeclarativeMeta, column: ColumnElement, value: Optional[datetime]) -> ColumnElement:
    if value is None:
        raise ReturnAll()

    return column >= value


class QueryFieldNameSystemQuery(query.Resource):
    class Meta:
        model = System

    name = fields.String(query_field_name="different_name")
    name_field = fields.String(query_field_name="other_name", table_column_name="name")
    from_ = fields.DateTimeFunction(start_time_function, query_field_name="from", attribute_name="created_at")
