from typing import Any, Callable, Dict, Optional

import pytest
from sqlalchemy.ext.declarative import DeclarativeMeta
from sqlalchemy.orm.query import Query, aliased
from sqlalchemy.sql.elements import ClauseElement, ColumnClause

from xcnt.sqlalchemy.search.parser.filters.exception import ModelFieldDoesNotExist
from xcnt.sqlalchemy.search.parser.flow_exception.return_all import ReturnAll
from xcnt.sqlalchemy.search.test import model
from xcnt.sqlalchemy.search.test import query as query_configurations


@pytest.fixture
def system(factories):
    return factories.System()


@pytest.fixture
def query_object():
    return model.System


@pytest.fixture
def query(session, query_object):
    def _query(
        query_filter: Dict[str, Any],
        clause_element_getter: Optional[Callable[[DeclarativeMeta, str], Optional[ClauseElement]]] = None,
    ) -> Query:
        query = session.query(query_object)
        configuration = query_configurations.FakeSystem(model=query_object, clause_element_getter=clause_element_getter)
        return configuration.apply(query, query_filter)

    return _query


@pytest.mark.parametrize("query_object", [model.System, aliased(model.System)])
def test_eq(query, system, factories):
    factories.System()
    db_query = query({"fake_alias": {"_eq": system.alias}})
    result = db_query.all()

    assert len(result) == 1
    result[0].uuid == system


def test_non_fake_ignored(query, system, factories):
    factories.System()
    with pytest.raises(ModelFieldDoesNotExist):
        query({"description": {"_eq": system.description}})


def test_overwrite_config(query, system, factories):
    def overwrite_function(model: DeclarativeMeta, column_name: str) -> Optional[ColumnClause]:
        return getattr(model, column_name, None)

    factories.System()
    db_query = query({"description": {"_eq": system.description}}, overwrite_function)
    result = db_query.all()

    assert len(result) == 1


def test_overwrite_with_ignore_field(query, system, factories):
    def overwrite_function(model: DeclarativeMeta, column_name: str) -> Optional[ColumnClause]:
        if column_name == "description":
            raise ReturnAll()

        return getattr(model, column_name, None)

    factories.System()
    db_query = query({"description": {"_eq": system.description}}, overwrite_function)
    result = db_query.all()

    assert len(result) == 2
