from typing import Any, Dict

import pytest
from sqlalchemy.orm.query import Query

from xcnt.sqlalchemy.search.test import model
from xcnt.sqlalchemy.search.test import query as query_configurations


@pytest.fixture
def system(factories):
    return factories.System()


@pytest.fixture
def query_object():
    return model.System


@pytest.fixture
def query(session, query_object):
    def _query(
        query_filter: Dict[str, Any],
    ) -> Query:
        query = session.query(query_object)
        configuration = query_configurations.IgnoredSystem(model=query_object)
        return configuration.apply(query, query_filter)

    return _query


def test_non_ignored_field(query, system, factories):
    factories.System()
    db_query = query({"alias": {"_eq": system.alias}})
    result = db_query.all()

    assert len(result) == 1


def test_overwrite_with_ignore_field(query, system, factories):
    factories.System()
    db_query = query({"description": {"_eq": system.description}})
    result = db_query.all()

    assert len(result) == 2
