from typing import Any, Dict

import pytest
from sqlalchemy.orm.query import Query

from xcnt.sqlalchemy.search.test import model
from xcnt.sqlalchemy.search.test import query as query_configurations


@pytest.fixture
def system(factories):
    return factories.System()


@pytest.fixture
def systems(factories):
    return factories.System.create_batch(20)


@pytest.fixture
def result_for(session):
    def _result_for(result_filter: Dict[str, Any]) -> Query:
        configuration = query_configurations.System()
        return configuration.generate(result_filter)

    return _result_for


@pytest.fixture
def query(session):
    def _query(query_filter: Dict[str, Any]) -> Query:
        query = session.query(model.Component)
        configuration = query_configurations.ComponentWithJoinedModels(model.Component)
        return configuration.apply(query, query_filter)

    return _query


def test_joined_model_filtering(query, system, factories):
    component = factories.Component(system=system)
    factories.Component()
    db_query = query({"system_alias": {"_eq": system.alias}})
    assert db_query.count() == 1
    assert db_query.first().uuid == component.uuid


def test_joined_model_filtering_round_robin(query, system, factories):
    components_in_system = [factories.Component(system=system), factories.Component(system=system)]
    factories.Component()

    db_query = query({"round_robin_component_code": {"_eq": components_in_system[0].code}})
    assert db_query.count() == 2
    all_uuids = [item.uuid for item in db_query]
    for component in components_in_system:
        assert component.uuid in all_uuids
