from typing import Any, Dict

import pytest
from sqlalchemy.orm.query import Query

from xcnt.sqlalchemy.search.test import model
from xcnt.sqlalchemy.search.test import query as query_configurations


@pytest.fixture
def component(factories):
    return factories.Component()


@pytest.fixture
def result_for(session):
    def _result_for(result_filter: Dict[str, Any]) -> Query:
        configuration = query_configurations.Component()
        return configuration.generate(result_filter)

    return _result_for


@pytest.fixture
def query(session):
    def _query(query_filter: Dict[str, Any]) -> Query:
        query = session.query(model.Component)
        configuration = query_configurations.Component(model.Component)
        return configuration.apply(query, query_filter)

    return _query


def test_nested_query_of_datapoint_retrieval(query, factories):
    component_1 = factories.Component()
    component_2 = factories.Component()
    factories.DatapointFloat(value=1.0, component=component_1)
    factories.DatapointFloat(value=1.1, component=component_2)

    where_filter_config = {"aggregate": {"float": {"aggregate_value": {"float": {"min": {"_eq": 1.0}}}}}}
    db_query = query(where_filter_config)
    assert db_query.all() == [component_1]
