from typing import Any, Dict

import pytest
from sqlalchemy.orm.query import Query

from xcnt.sqlalchemy.search.test import model
from xcnt.sqlalchemy.search.test import query as query_configurations


@pytest.fixture
def system(factories):
    return factories.System()


@pytest.fixture
def systems(factories):
    return factories.System.create_batch(20)


@pytest.fixture
def result_for(session):
    def _result_for(result_filter: Dict[str, Any]) -> Query:
        configuration = query_configurations.System()
        return configuration.generate(result_filter)

    return _result_for


@pytest.fixture
def query(session):
    def _query(query_filter: Dict[str, Any]) -> Query:
        query = session.query(model.System)
        configuration = query_configurations.System(model.System)
        return configuration.apply(query, query_filter)

    return _query


def test_n_m_relationship_count(query, system, factories):
    component_model = factories.ComponentModel()
    [
        factories.Component(system=system, component_model=component_model),
        factories.Component(system=system, component_model=component_model),
    ]
    db_query = query({"components": {"componentModel": {"_oneOf": [component_model.uuid]}}})
    assert db_query.count() == 1
