from typing import Any, Dict
from uuid import uuid4

import pytest
from sqlalchemy.orm.query import Query

from xcnt.sqlalchemy.search.test import model
from xcnt.sqlalchemy.search.test import query as query_configurations


@pytest.fixture
def component(factories):
    return factories.Component()


@pytest.fixture
def result_for(session):
    def _result_for(result_filter: Dict[str, Any]) -> Query:
        configuration = query_configurations.Component()
        return configuration.generate(result_filter)

    return _result_for


@pytest.fixture
def query(session):
    def _query(query_filter: Dict[str, Any]) -> Query:
        query = session.query(model.Component)
        configuration = query_configurations.Component(model.Component)
        return configuration.apply(query, query_filter)

    return _query


def test_nested_eq(query, component, factories):
    factories.Component()
    db_query = query({"system": {"alias": {"_eq": component.system.alias}}})
    result = db_query.all()
    assert len(result) == 1
    assert result[0].uuid == component.uuid


def test_nested_eq_backreference(query, component, factories):
    component_2 = factories.Component(system=component.system)
    factories.Component()
    db_query = query({"system": {"components": {"_oneOf": [uuid4()]}, "alias": {"_eq": component_2.system.alias}}})
    result = db_query.all()
    assert len(result) == 0


def test_nested_eq_backreference_double_join(query, component, factories):
    component_2 = factories.Component(system=component.system)
    factories.Component()
    db_query = query(
        {"system": {"components": {"_oneOf": [component_2.uuid]}, "alias": {"_eq": component.system.alias}}}
    )
    result = db_query.all()
    assert len(result) == 2


def test_nested_morphed_eq(query, component, factories):
    factories.Component()
    db_query = query({"componentModel": {"alias": {"_eq": component.component_model.alias}}})
    result = db_query.all()
    assert len(result) == 1
    assert result[0].uuid == component.uuid


def test_nested_eq_backreference_triple_join(query, component, factories):
    component_2 = factories.Component(system=component.system)
    factories.Component()
    db_query = query(
        {
            "system": {
                "components": {
                    "_oneOf": [component_2.uuid],
                    "componentModel": {"alias": {"_eq": component_2.component_model.alias}},
                }
            },
        }
    )
    result = db_query.all()
    assert len(result) == 2


def test_nested_model_name(query, component, factories):
    component = factories.Component(system=component.system)
    other_component = factories.Component()
    factories.DatapointFloat(component=component, value=3.0)
    factories.DatapointFloat(value=4.0, component=other_component)
    db_query = query({"float_datapoints_via_other_name": {"value": {"_eq": 3.0}}})
    assert db_query.count() == 1
    assert component in db_query.all()
