import pytest

from xcnt.sqlalchemy.search import fields, query
from xcnt.sqlalchemy.search.test import query as query_configurations


class LocalComponentModelQuery(query.Resource):
    pass


class LocalComponentQuery(query.Resource):
    component_model = fields.Nested("LocalComponentModelQuery")


class LocalComponentQueryLookupFailure(query.Resource):
    component_model = fields.Nested("NotHereComponentModelQuery")


class LocalComponentQueryLookupFailureImportPath(query.Resource):
    component_model = fields.Nested("not.a.valid.import.path.for.Query")


@pytest.fixture
def patched_relative_system_query():
    components_nested = query_configurations.System.components
    query_configurations.System.components = fields.Nested("..component.ComponentQuery")
    yield query_configurations.System
    query_configurations.System.components = components_nested  # type: ignore


def test_nested_lookup_absolute():
    assert query_configurations.System.components.nested_resource == query_configurations.Component


def test_nested_lookup_relative_back_reference(patched_relative_system_query):
    assert patched_relative_system_query.components.nested_resource == query_configurations.Component


def test_local_component_lookup():
    assert LocalComponentQuery.component_model.nested_resource == LocalComponentModelQuery


def test_lookup_failure_local():
    with pytest.raises(ValueError):
        LocalComponentQueryLookupFailure.component_model.nested_resource


def test_lookup_failure_import_path():
    with pytest.raises(ValueError):
        LocalComponentQueryLookupFailureImportPath.component_model.nested_resource
