from xcnt.sqlalchemy.search.test import model
from xcnt.sqlalchemy.search.test import query as query_configurations


def test_field_group_nested_object(session, factories):
    component = factories.Component()
    other_component = factories.Component()
    factories.DatapointFloat.create(value=3.0, component=component)
    factories.DatapointFloat.create(value=2.0, component=other_component)
    query = session.query(model.Component)
    configuration = query_configurations.Component(model.Component)
    query_filter = {"aggregate": {"float": {"value": {"_eq": 3.0}}}}
    db_query = configuration.apply(query, query_filter)

    assert db_query.count() == 1
    assert component in db_query.all()


def test_field_group_scalar_object(session, factories):
    component = factories.Component()
    other_component = factories.Component()
    factories.DatapointFloat.create(value=3.0, component=component)
    factories.DatapointFloat.create(value=2.0, component=other_component)
    query = session.query(model.Component)
    configuration = query_configurations.Component(model.Component)
    query_filter = {
        "component_attributes": {"code": {"_eq": component.code}},
        "aggregate": {"float": {"value": {"_in": [3.0, 2.0]}}},
    }
    db_query = configuration.apply(query, query_filter)

    assert db_query.count() == 1
    assert component in db_query.all()


def test_field_group_scalar_object_no_return(session, factories):
    component = factories.Component()
    other_component = factories.Component()
    factories.DatapointFloat.create(value=3.0, component=component)
    factories.DatapointFloat.create(value=2.0, component=other_component)
    query = session.query(model.Component)
    configuration = query_configurations.Component(model.Component)
    query_filter = {
        "component_attributes": {"code": {"_eq": component.code}},
        "aggregate": {"float": {"value": {"_in": [2.0]}}},
    }
    db_query = configuration.apply(query, query_filter)

    assert db_query.count() == 0


def test_field_group_with_or_statement(session, factories):
    component = factories.Component()
    other_component = factories.Component()
    factories.Component()
    query = session.query(model.Component)
    configuration = query_configurations.Component(model.Component)
    query_filter = {
        "component_or_attributes": {
            "code": {"_eq": component.code},
            "other_code": {"_eq": other_component.code},
        }
    }
    db_query = configuration.apply(query, query_filter)

    assert db_query.count() == 2


def test_field_group_with_or_statement_and_nested_and(session, factories):
    component = factories.Component()
    other_component = factories.Component()

    query = session.query(model.Component)
    configuration = query_configurations.Component(model.Component)
    query_filter = {
        "component_or_attributes": {
            "code": {"_eq": component.code, "_neq": component.code},
            "other_code": {"_eq": other_component.code, "_neq": other_component.code},
        }
    }
    db_query = configuration.apply(query, query_filter)

    assert db_query.count() == 0


def test_multi_use_field_group_does_not_overwrite_base_resource(session, factories):
    system = factories.System()
    component_model = factories.ComponentModel()
    factories.Component(system=system, component_model=component_model)
    factories.Component(system=factories.System(), component_model=factories.ComponentModel())
    assert system.alias != component_model.alias

    query_filter = {
        "alias_group": {"alias": {"_eq": system.alias}},
        "components": {"component_model": {"alias_group": {"alias": {"_eq": component_model.alias}}}},
    }
    query = session.query(model.System)
    configuration = query_configurations.System(model.System)
    db_query = configuration.apply(query, query_filter)
    assert db_query.count() == 1
    assert db_query.first().uuid == system.uuid

    system_field_group = configuration._fields["alias_group"]
    component_model_configuration = query_configurations.ComponentModel()
    component_model_field_group = component_model_configuration._fields["alias_group"]
    assert system_field_group.resource is configuration
    assert isinstance(component_model_field_group.resource, query_configurations.ComponentModel)

    system_alias_field = system_field_group._fields["alias"]
    component_model_alias_field = component_model_field_group._fields["alias"]
    assert system_alias_field.resource is configuration
    assert isinstance(component_model_alias_field.resource, query_configurations.ComponentModel)
