from datetime import datetime, timedelta
from typing import Any, Dict

import pytest
from sqlalchemy.orm.query import Query

from xcnt.sqlalchemy.search.test import model
from xcnt.sqlalchemy.search.test import query as query_configurations


@pytest.fixture
def system(factories):
    return factories.System()


@pytest.fixture
def query_object():
    return model.System


@pytest.fixture
def query(session, query_object):
    def _query(
        query_filter: Dict[str, Any],
    ) -> Query:
        query = session.query(query_object)
        configuration = query_configurations.QueryFieldNameSystem(model=query_object)
        return configuration.apply(query, query_filter)

    return _query


def test_query_field_name(query, system, factories):
    factories.System()
    db_query = query({"different_name": {"_eq": system.name}})
    result = db_query.all()

    assert len(result) == 1


def test_query_field_name_attribute_name_field(query, system, factories):
    factories.System()
    db_query = query({"other_name": {"_eq": system.name}})
    result = db_query.all()

    assert len(result) == 1


def test_query_field_name_and_attribute_name_in_function_field(session, query, system, factories):
    systems = factories.System.create_batch(3)
    for item in systems:
        item.created_at = datetime.utcnow() - timedelta(hours=30)

    session.commit()

    db_query = query({"from": system.created_at.isoformat()})
    result = db_query.all()

    assert len(result) == 1
    assert result[0].uuid == system.uuid
