import base64
from typing import Any, Dict

import pytest
from sqlalchemy.orm.query import Query

from xcnt.sqlalchemy.search.test import model
from xcnt.sqlalchemy.search.test import query as query_configurations


@pytest.fixture
def datapoint_string(factories):
    return factories.DatapointString()


@pytest.fixture
def datapoint_float(factories):
    return factories.DatapointFloat()


@pytest.fixture
def datapoint_blob(factories):
    return factories.DatapointBlob()


@pytest.fixture
def result_for(session):
    def _result_for(result_filter: Dict[str, Any]) -> Query:
        configuration = query_configurations.Datapoint()
        return configuration.generate(result_filter)

    return _result_for


@pytest.fixture
def query(session):
    def _query(query_filter: Dict[str, Any]) -> Query:
        query = session.query(model.DatapointBase)
        configuration = query_configurations.Datapoint(model.DatapointBase)
        return configuration.apply(query, query_filter)

    return _query


def test_eq(query, datapoint_float, datapoint_string, factories):
    factories.DatapointFloat()
    db_query = query({"float": {"_eq": datapoint_float.value}})
    result = db_query.all()

    assert len(result) == 1
    result[0].uuid == datapoint_float.uuid


def test_eq_ignore_invalid_attribute_name(session, datapoint_float, datapoint_string, factories):
    query = session.query(model.DatapointString)
    configuration = query_configurations.Datapoint(model.DatapointString)
    query_filter = {"float": {"_eq": datapoint_float.value}}
    db_query = configuration.apply(query, query_filter)
    result = db_query.all()

    assert len(result) == 0


def test_eq_valid_attribute_name(session, datapoint_float, datapoint_string, factories):
    query = session.query(model.DatapointFloat)
    configuration = query_configurations.Datapoint(model.DatapointFloat)
    query_filter = {"float": {"_eq": datapoint_float.value}}
    db_query = configuration.apply(query, query_filter)
    result = db_query.all()

    assert len(result) == 1


def test_blob_eq(session, datapoint_string, datapoint_float, datapoint_blob, factories):
    factories.DatapointBlob.create_batch(2)
    query = session.query(model.DatapointBlob)
    configuration = query_configurations.Datapoint(model.DatapointBlob)
    query_filter = {"blob": {"_eq": base64.b64encode(datapoint_blob.value).decode("utf-8")}}
    db_query = configuration.apply(query, query_filter)
    result = db_query.all()

    assert len(result) == 1


def test_blob_eq_invalid(session, datapoint_string, datapoint_float, datapoint_blob, factories):
    factories.DatapointBlob.create_batch(2)
    query = session.query(model.DatapointBlob)
    configuration = query_configurations.Datapoint(model.DatapointBlob)
    query_filter = {"blob": {"_eq": "asdfasdf"}}
    db_query = configuration.apply(query, query_filter)
    result = db_query.all()

    assert len(result) == 0
