import pytest

from xcnt.sqlalchemy.search import fields
from xcnt.sqlalchemy.search.parser.filters import ModelFieldDoesNotExist
from xcnt.sqlalchemy.search.test import model
from xcnt.sqlalchemy.search.test.query import System


class SystemQueryTableName(System):
    system_name = fields.String(table_column_name="name")


class SystemQueryInvalidTableName(System):
    system_name = fields.String(table_column_name="invalid_column_name")


@pytest.fixture
def system(factories):
    return factories.System()


@pytest.fixture
def systems(factories):
    return factories.System.create_batch(20)


def test_system_table_column_success(session, system, systems):
    query = session.query(model.System)
    configuration = SystemQueryTableName(model.System)
    query_filter = {"system_name": {"_eq": system.name}}
    db_query = configuration.apply(query, query_filter)
    result = db_query.all()

    assert len(result) == 1
    assert result[0].name == system.name


def test_system_table_column_invalid_column(session, system, systems):
    query = session.query(model.System)
    configuration = SystemQueryInvalidTableName(model.System)
    query_filter = {"system_name": {"_eq": system.name}}
    with pytest.raises(ModelFieldDoesNotExist):
        configuration.apply(query, query_filter)
