from datetime import timedelta

import pytest
from isodate import Duration

from xcnt.sqlalchemy.search.type_conversion.timedelta import convert_to_timedelta


def test_convert_from_iso_format():
    assert convert_to_timedelta("PT14H30M30S") == timedelta(hours=14, minutes=30, seconds=30)


def test_convert_with_isoformat_day():
    assert convert_to_timedelta("P20DT30M") == timedelta(days=20, minutes=30)


def test_convert_with_isoformat_year_and_month():
    assert convert_to_timedelta("P10Y1MT30M") == Duration(minutes=30, months=1, years=10)


def test_convert_one_month_and_one_minute():
    assert convert_to_timedelta("P1MT1M") == Duration(minutes=1, months=1)


def test_convert_from_naive_config():
    assert convert_to_timedelta("121m") == timedelta(hours=2, minutes=1)


def test_convert_with_timedelta_object():
    assert convert_to_timedelta(timedelta(seconds=2)) == timedelta(seconds=2)


def test_convert_with_int():
    assert convert_to_timedelta(3) == timedelta(seconds=3)


def test_convert_with_float():
    assert convert_to_timedelta(5.5) == timedelta(seconds=5, milliseconds=500)


class ThisIsNotParseable:
    pass


def test_conversion_with_invalid_type():
    with pytest.raises(ValueError):
        convert_to_timedelta(ThisIsNotParseable())


def test_conversion_with_invalid_string():
    with pytest.raises(ValueError):
        convert_to_timedelta("This is not parseable")
