from datetime import timedelta
from typing import Any

import pytimeparse
from isodate import ISO8601Error, parse_duration


def convert_to_timedelta(value: Any) -> timedelta:
    if isinstance(value, str):
        try:
            return parse_duration(value)
        except ISO8601Error:
            total_seconds = pytimeparse.parse(value)
            if total_seconds is None:
                raise ValueError(f"Cannot convert {value} to timespan")
            return timedelta(seconds=total_seconds)
    elif isinstance(value, timedelta):
        return value
    elif isinstance(value, (int, float)):
        return timedelta(seconds=value)
    else:
        raise ValueError(f"Cannot convert {value} to timespan")
